import java.io.*;

/**
 * The superclass which defines shared instance variables and methods used by
 * both Encoder and Decoder.
 */
public class Endec {
    protected Image im;
    protected ArithmeticCoder ar;
    
    protected Coords[] contextCoords;
    protected Context[] contexts;
    
    
    /**
     * Creates an array of all the possible contexts as specified by their ordered pairs.
     * This consists of 2^((bit-depth)^(# of coordinates)) since each ordered pair can take on
     * 2^(bit-depth) values. Every context is initialized so that the count of all possible values
     * is equal (equal probabilities).
     */
    protected void createContextArray() {
        int possibleValues = (int)Math.pow(2, im.bitDepth);
        int totalContexts = (int)Math.pow(possibleValues, contextCoords.length);
        contexts = new Context[totalContexts];
        for(int i=0; i < contexts.length; i++) {
            int[] startCount = new int[possibleValues];
            // set start count for all possible values to 1
            for(int j=0; j < startCount.length; j++)
                startCount[j] = 1;
            contexts[i] = new Context(startCount);
            contexts[i].indexNum = i;
        }
    }

    /**
     * Finds the correct context to be applied to the pixel currently being encoded/decoded
     * in the pixel array.
     * 
     * @param i The x coordinate of the location of the pixel
     * @param j The y coordinate of the location of the pixel
     * @return The context to be used for this pixel
     */
    protected Context findContext(int i, int j) {
        int contextNum = 0;
        int possibleValues = (int)Math.pow(2, im.bitDepth);

        // iterate through all the coordinates defined in the context
        for (int k=0; k < contextCoords.length; k++) {

            int pixel;
            int x = contextCoords[k].x;
            int y = contextCoords[k].y;

            // if one of the coordinates lies off the edge of the bitmap, assume it is the dominant color
            if (x < 0 && i+x < 0 || // off the left edge
                x > 0 && i+x > im.width-1 || // off the right edge
                y < 0 && j+y < 0) // off the top edge
                pixel = im.dominantColor;

            else // normal case
                pixel = im.pixels[i+x][j+y];

            /* 
             * The context index number depends on the pixel value in each ordered pair position
             * and also depends on the bit-depth. The number of digits in the index number is
             * the number of ordered pairs. The pixel in each ordered pair position equals the 
             * digit at each position in the number. 2^(bit-depth) is the base of the number.
             * 
             * Ex. Suppose the bit-depth is 8 and the contexts consist of 3 ordered pairs. If one context
             * has pixel values 0, 7, and 3 in the first, second, and third ordered pairs respectively,
             * then the index number is 073 with a base of 2^8. This evaluates to:
             *   (0 * 256^2) + (7 * 256^1) + (3 * 256^0) = 1792
             */
            contextNum += pixel * (int)Math.pow(possibleValues, (contextCoords.length-1) - k);
        }
        return contexts[contextNum];
    }

}