import java.util.*;
import java.io.*;

/**
 * Main class that runs the arithmetically encodes and decodes an image.
 * 
 * Usage:
 * java ImageCoder inputImage [contextFile] encodedFile outputImage
 * 
 * where
 * inputImage = the image to encode
 * contextFile (optional) = text file representing the type of context to use (see ContextReader for details)
 *                          if no contextFile is supplied, the following default context is used: AA
 *                                                                                                A?
 * encodedFile = the file to write the encoded result to (will also be read from for the decoding process)
 * outputImage = the file to write the decoded image to
 *
 * @author Amanda Askew & Dane Barney
 * @version 2003.09.11
 */
public class ImageCoder {
    private static final String USAGE = "Usage:\n"+
                                        "java ImageCoder inputImage [contextFile] encodedFile outputImage";

    public static void main(String[] args) {
        if(args.length != 3 && args.length != 4) {
            System.out.println("Invalid syntax. "+USAGE);
            return;
        }
        
        try {
            
            // interpret command line args
            String input = args[0];
            String contextFile = args.length == 4 ? args[1] : "";
            String encoded = args.length == 4 ? args[2] : args[1];
            String output = args.length == 4 ? args[3] : args[2];
            
            // read image from file
            Image im = new Image(input, true);
            
            // get context coordinates from file
            Coords[] contextCoords = ContextReader.getContext(contextFile);
            System.out.print("Context coordinates: ");
            for(int i=0; i < contextCoords.length; i++) {
                Coords cd = contextCoords[i];
                System.out.print("("+cd.x+","+cd.y+") ");
            }
            System.out.println();
            
            
            /**************************************
             *       Begin Encoding Process       *
             **************************************/
            System.out.println("##### Compressing "+input+" to "+encoded+" #####");

            // create encoder object, which will handle all writing to encoded file
            Encoder enc = new Encoder(im, encoded, contextCoords);
            
            // encode image pixels to file
            enc.encode();
            
            // print results
            int bef = im.filesize;
            int aft = enc.out.bytesWritten;
            System.out.println("Original filesize: "+bef+" bytes\nCompressed filesize: "+aft+" bytes");
            System.out.println("Compression ratio: "+(double)bef/aft+":1, "+(double)(bef-aft)/bef*100+"%");
            System.out.println("Compression rate: "+(double)enc.encodedBytes*8/(im.width*im.height)+" bits/pixel (Originally "+im.bitDepth+" bits/pixel)");
            System.out.println("Finished encoding.");
            System.out.println();
            
            // free memory
            im = null;
            enc = null;
            // could instead use the same image header and contexts for decoder,
            // completely bypassing the need for an intermediate encoded file


            /**************************************
             *       Begin Decoding Process       *
             **************************************/
            System.out.println("##### Decompressing "+encoded+" to "+output+" #####");
            
            // create decoder object, which will handle all reading from encoded file
            Decoder dec = new Decoder(encoded);
            
            // decode image pixels from file
            im = dec.decode();
            
            // save image to file
            im.save(output);
            
            System.out.println("Finished decoding.");
                        
        } catch(Throwable t) {
            System.out.println(t);
            t.printStackTrace(System.out);
        }
    }
    
}